<?php

use Zen\Payment\Api;

require_once 'zen' . DIRECTORY_SEPARATOR . 'ZenHelper.php';
require_once 'zen' . DIRECTORY_SEPARATOR . 'payment-core' . DIRECTORY_SEPARATOR . 'src' . DIRECTORY_SEPARATOR . 'Util.php';
require_once 'zen' . DIRECTORY_SEPARATOR . 'payment-core' . DIRECTORY_SEPARATOR . 'src' . DIRECTORY_SEPARATOR . 'Paywall.php';
require_once 'zen' . DIRECTORY_SEPARATOR . 'payment-core' . DIRECTORY_SEPARATOR . 'src' . DIRECTORY_SEPARATOR . 'Api.php';
require_once 'zen' . DIRECTORY_SEPARATOR . 'payment-core' . DIRECTORY_SEPARATOR . 'src' . DIRECTORY_SEPARATOR . 'CartData.php';
require_once 'zen' . DIRECTORY_SEPARATOR . 'payment-core' . DIRECTORY_SEPARATOR . 'src' . DIRECTORY_SEPARATOR . 'Notification.php';
require_once 'zen' . DIRECTORY_SEPARATOR . 'payment-core' . DIRECTORY_SEPARATOR . 'src' . DIRECTORY_SEPARATOR . 'Validate.php';

class zen extends base
{
	public $code;
	public $title;
	public $description;
	public $enabled;

	/**
	 * Zen constructor.
	 */
	public function __construct()
	{
		global $order;

		$this->code = 'zen';
		$this->title = MODULE_PAYMENT_ZEN_TEXT_TITLE;
		$this->description = MODULE_PAYMENT_ZEN_TEXT_DESCRIPTION;
		$this->sort_order = MODULE_PAYMENT_ZEN_SORT_ORDER;
		$this->enabled = MODULE_PAYMENT_ZEN_STATUS === 'True';

		if((int) MODULE_PAYMENT_ZEN_ORDER_STATUS_ID > 0) {
			$this->order_status = MODULE_PAYMENT_ZEN_ORDER_STATUS_ID;
		}

		if(is_object($order)) {
			$this->update_status();
		}
	}

	/**
	 * @param $orderId
	 *
	 * @return string
	 */
	public function admin_notification($orderId)
	{

		if(!defined('MODULE_PAYMENT_ZEN_STATUS')) {

			return;
		}

		if(!ZenHelper::getTransactionByOrderId($orderId)->fields['transaction_id']) {
			return;
		}

		$orderDbObject = ZenHelper::getOrderById($orderId);
		$order = $orderDbObject->fields;

		$output = '<td><table>';
		$output .= '<tr style="background-color : #cccccc; border-style : dotted;">';

		$refundsDbObject = ZenHelper::getRefundHistoryForOrder($orderId);

		$refunded = $this->getRefundedAmountForOrder($orderId);

		if((float) $order['order_total'] > (float) $refunded) {

			$output .= $this->createRefundForm();
		}

		if($refundsDbObject->RecordCount()) {
			$output .= $this->createRefundHistoryTable($refundsDbObject, $refunded);
		}

		$output .= '</tr>';
		$output .= '</table></td>';

		return $output;
	}

	/**
	 * @param $orderId
	 *
	 * @return string
	 */
	public function getRefundedAmountForOrder($orderId)
	{
		global $db;

		$sql = 'SELECT SUM(`refund_amount`) AS `refunded` FROM ' . ZenHelper::ZEN_REFUNDS_HISTORY_TABLE_NAME
			. ' WHERE `order_id` = :id';
		$sql = $db->bindVars($sql, ':id', $orderId, 'integer');
		$refundsAmountSum = $db->Execute($sql);

		return $refundsAmountSum->fields['refunded'];
	}

	/**
	 * Update status function
	 */
	public function update_status()
	{
		global $db;
		global $order;
		// check zone
		if($this->enabled && (int) $this->zone > 0 && isset($order->billing['country']['id'])) {
			$check_flag = false;
			$sql = 'SELECT zone_id
              FROM ' . TABLE_ZONES_TO_GEO_ZONES
				. ' WHERE geo_zone_id = :zoneId
              AND zone_country_id = :countryId
              ORDER BY zone_id';
			$sql = $db->bindVars($sql, ':zoneId', $this->zone, 'integer');
			$sql = $db->bindVars($sql, ':countryId', $order->billing['country']['id'], 'integer');
			$check = $db->Execute($sql);
			while(!$check->EOF) {
				if($check->fields['zone_id'] < 1) {
					$check_flag = true;
					break;
				} elseif($check->fields['zone_id'] === $order->billing['zone_id']) {
					$check_flag = true;
					break;
				}
				$check->MoveNext();
			}
		}
		if($check_flag === false) {
			$this->enabled = false;
		}
		if($order->info['total'] == 0) {
			$this->enabled = false;
		}
		if(!MODULE_PAYMENT_ZEN_APIHASH or !strlen(MODULE_PAYMENT_BITPAY_APIKEY) or
			!MODULE_PAYMENT_ZEN_API_SECRET or !strlen(MODULE_PAYMENT_ZEN_API_SECRET) or
			!MODULE_PAYMENT_ZEN_MERCHANTEMAIL or !strlen(MODULE_PAYMENT_ZEN_MERCHANTEMAIL)) {
			$this->enabled = false;
		}
	}

	/**
	 * Check the user input submitted on checkout_payment.php with javascript (client-side).
	 * Examples: validate credit card number, make sure required fields are filled in.
	 *
	 * @return bool
	 */
	public function javascript_validation()
	{
		return false;
	}

	/**
	 * @return array
	 */
	public function selection()
	{
		return [
			'id'     => $this->code,
			'module' => $this->title,
		];
	}

	/**
	 * Pre confirmation checks (ie, check if credit card information is right before sending the info to the payment server.
	 *
	 * @return bool
	 */
	public function pre_confirmation_check()
	{
		return false;
	}

	/**
	 * Functions to execute before displaying the checkout confirmation page.
	 *
	 * @return bool
	 */
	public function confirmation()
	{
		return false;
	}

	/**
	 * Functions to execute before finishing the form
	 * Examples: add extra hidden fields to the form.
	 *
	 * @return bool
	 */
	public function process_button()
	{
		return false;
	}

	/**
	 * @return bool
	 */
	public function before_process()
	{

		return false;
	}

	/**
	 * @return bool
	 */
	public function after_process()
	{
		global $insert_id;
		global $order;

		ZenHelper::updateOrderStatus($insert_id, MODULE_PAYMENT_ZEN_UNPAID_STATUS_ID);

		$paywall = new \Zen\Payment\Paywall(
		    \Zen\Payment\Util::ENVIRONMENT_PRODUCTION,
			MODULE_PAYMENT_ZEN_PAYWALL_SECRET,
			MODULE_PAYMENT_ZEN_TERMINAL_UUID
        );

		$cartData = new \Zen\Payment\CartData();

		$currencyValue = (float) $order->info['currency_value'];

		foreach($this->getItemsArray($order) as $key => $value) {

			$cartData->addItem(
				$value['name'],
				\Zen\Payment\Util::convertAmountToFractional($value['price'] * $currencyValue),
				$value['qty'],
				\Zen\Payment\Util::convertAmountToFractional($value['amount'] * $currencyValue)
			);
		}

		$total = \Zen\Payment\Util::convertAmountToFractional((float) $order->info['total'] * $currencyValue);
		$cartData->setAmount($total);

		$form = $paywall->buildOrderForm(
			\Zen\Payment\Paywall::prepareData(
				$total,
				$order->info['currency'],
				$insert_id,
				$order->customer['firstname'],
				$order->customer['lastname'],
				$order->customer['email_address'],
				zen_href_link(FILENAME_CHECKOUT_SUCCESS, '', 'SSL'),
				zen_href_link(FILENAME_ACCOUNT_HISTORY_INFO, 'order_id=' . $insert_id, 'SSL'),
				zen_href_link(FILENAME_ACCOUNT_HISTORY_INFO, 'order_id=' . $insert_id, 'SSL'),
				zen_href_link('zenipn.php', '', 'SSL'),
				$cartData->getCartData(MODULE_PAYMENT_ZEN_TEXT_ROUND)
			)
		);

		if(!$form) {
			return false;
		}

		$_SESSION['cart']->reset(true);

		echo MODULE_PAYMENT_ZEN_TEXT_REDIRECT
			. '<br/>'
			. $form
			. '<script type="text/javascript">'
			. 'document.getElementById(\'submit-payment-form\').style.display = "none";'
			. 'document.getElementById(\'submit-payment-form\').click();'
			. '</script>';
		die();
	}

	/**
	 * @param $order
	 *
	 * @return array
	 */
	public function getItemsArray($order)
	{
		$items = [];
		foreach($order->products as $orderItem) {
			$items[] = [
				'name'   => $orderItem['name'],
				'amount' => $orderItem['final_price'] * $orderItem['qty'],
				'qty'    => (integer) $orderItem['qty'],
				'price'  => $orderItem['final_price'],
			];
		}

		$orderDetails = $order->info;
		$items[] = [
			'name'   => $orderDetails['shipping_method'],
			'amount' => $orderDetails['shipping_cost'],
			'qty'    => 1,
			'price'  => $orderDetails['shipping_cost'],
		];

		if($orderDetails['tax'] > 0) {
			$items[] = [
				'name'   => MODULE_PAYMENT_ZEN_TEXT_TAX,
				'amount' => $orderDetails['tax'],
				'qty'    => 1,
				'price'  => $orderDetails['tax'],
			];
		}

		$discountAmount = $_SESSION['order_summary']['credits_applied'];
		if($discountAmount > 0) {
			$items[] = [
				'name'   => MODULE_PAYMENT_ZEN_TEXT_ROUND,
				'amount' => -$discountAmount,
				'qty'    => 1,
				'price'  => -$discountAmount,
			];
		}

		return $items;
	}

	/**
	 * @param $orderId
	 *
	 * @return bool
	 */
	public function _doRefund($orderId)
	{
		global $messageStack;

		if($_SERVER['REQUEST_METHOD'] !== 'POST') {
			$messageStack->add_session(MODULE_PAYMENT_ZEN_TEXT_REFUND_INVALID_REQUEST_METHOD);

			return;
		}
		if(isset($_POST['fullrefund']) && !isset($_POST['reffullconfirm'])) {

			$messageStack->add_session(MODULE_PAYMENT_ZEN_TEXT_REFUND_NOT_CONFIRMED);

			return;
		}

		$amount = str_replace(',', '.', $_POST['refund_amount']);

		if(!is_numeric($amount) && isset($_POST['partialrefund'])) {
			$messageStack->add_session(MODULE_PAYMENT_ZEN_TEXT_REFUND_AMOUNT_NOT_NUMERIC);

			return;
		}

		$orderDbObject = ZenHelper::getOrderById($orderId);
		$order = $orderDbObject->fields;
		$currencyValue = (float) $order['currency_value'];

		$refundedAmountForOrder = $this->getRefundedAmountForOrder($orderId);

		if(isset($_POST['partialrefund'])) {
			$amount = ZenHelper::getValidAmount($amount);

			if(((float) $refundedAmountForOrder + (float) $amount) > (float) $order['order_total']) {
				$messageStack->add_session(MODULE_PAYMENT_ZEN_TEXT_REFUND_AMOUNT_HIGHER_THAN_ORDER);

				return;
			}
		} elseif(isset($_POST['fullrefund'])) {
			$amount = round((ZenHelper::getValidAmount($order['order_total'])
					- ZenHelper::getValidAmount($refundedAmountForOrder)) * $currencyValue, 2);
		} else {
			$messageStack->add_session(MODULE_PAYMENT_ZEN_TEXT_REFUND_INVALID_REQUEST_PARAMS);

			return;
		}

		$transaction = ZenHelper::getTransactionByOrderId($orderId);
		$transactionId = $transaction->fields['transaction_id'];
		$api = new Api(
			MODULE_PAYMENT_ZEN_TERMINAL_API_KEY
		);

		$refund = $api->createRefund(
			\Zen\Payment\Util::ENVIRONMENT_PRODUCTION,
			$amount,
			$transactionId,
			$order['currency'],
			$orderId
		);

		if(!$refund['success']) {
			if(isset($refund['data']['zenError']['message']) && $refund['data']['zenError']['message']) {
				$messageStack->add_session('Refund error: ' . explode(';', $refund['data']['zenError']['message'])[0]);

				return;
			} else {
				$messageStack->add_session('Refund error');

				return;
			}
		}

		$messageStack->add_session(MODULE_PAYMENT_ZEN_TEXT_REFUND_SUCCESS . $amount . ' ' . $order['currency'], 'success');

		return true;
	}

	/**
	 * If an error occurs with the process, output error messages here.
	 *
	 * @return bool
	 */
	public function get_error()
	{
		return false;
	}

	/**
	 * Check if module is installed (Administration Tool).
	 *
	 * @return int
	 */
	public function check()
	{
		global $db;
		if(!isset($this->_check)) {
			$check_query = $db->Execute('SELECT configuration_value FROM ' . TABLE_CONFIGURATION
				. " WHERE configuration_key = 'MODULE_PAYMENT_ZEN_STATUS'");
			$this->_check = $check_query->RecordCount();
		}

		return $this->_check;
	}

	/**
	 * install module by inserting all module config keys to configuration table.
	 *
	 * @return string
	 */
	public function install()
	{
		global $db, $messageStack;

		if(defined('MODULE_PAYMENT_ZEN_STATUS')) {
			$messageStack->add_session('Zen module already installed.', 'error');
			$urlParams = [
				'set'    => 'payment',
				'module' => 'zen',
			];
			zen_redirect(zen_href_link(FILENAME_MODULES, http_build_query($urlParams)));

			return 'failed';
		}

		$db->Execute('INSERT INTO ' . TABLE_CONFIGURATION . ' (configuration_title, configuration_key,
         configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) '
			. "VALUES ('Enable ZEN module', 'MODULE_PAYMENT_ZEN_STATUS', 'True', 
            'Enable easy payments via blik, pbl, etc.', '6', '0', 
            'zen_cfg_select_option(array(\'True\', \'False\'), ', now());");

		$db->Execute('INSERT INTO ' . TABLE_CONFIGURATION . ' (configuration_title, configuration_key, 
        configuration_value, configuration_description, configuration_group_id, sort_order, date_added) '
			. "VALUES ('Terminal UUID', 'MODULE_PAYMENT_ZEN_TERMINAL_UUID', '', '', '6', '0', now());");

		$db->Execute('INSERT INTO ' . TABLE_CONFIGURATION . ' (configuration_title, configuration_key, 
        configuration_value, configuration_description, configuration_group_id, sort_order, date_added) '
			. "VALUES ('Paywall Secret', 'MODULE_PAYMENT_ZEN_PAYWALL_SECRET', '', '', '6', '0', now());");

		$db->Execute('INSERT INTO ' . TABLE_CONFIGURATION . ' (configuration_title, configuration_key, 
        configuration_value, configuration_description, configuration_group_id, sort_order, date_added) '
			. "VALUES ('IPN Secret', 'MODULE_PAYMENT_ZEN_IPN_SECRET', '', '', '6', '0', now());");

		$db->Execute('INSERT INTO ' . TABLE_CONFIGURATION . ' (configuration_title, configuration_key, 
        configuration_value, configuration_description, configuration_group_id, sort_order, date_added) '
			. "VALUES ('Terminal API key', 'MODULE_PAYMENT_ZEN_TERMINAL_API_KEY', '', '', '6', '0', now());");

		$db->Execute('INSERT INTO ' . TABLE_CONFIGURATION . ' (configuration_title, configuration_key, 
        configuration_value, configuration_description, configuration_group_id, sort_order, set_function, 
        use_function, date_added) '
			. "VALUES ('Unpaid Order Status', 'MODULE_PAYMENT_ZEN_UNPAID_STATUS_ID', '"
			. intval(DEFAULT_ORDERS_STATUS_ID) . "', 'Automatically set the status of unpaid orders to this value.', 
            '6', '0', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");

		$db->Execute('INSERT INTO ' . TABLE_CONFIGURATION . ' (configuration_title, configuration_key, 
        configuration_value, configuration_description, configuration_group_id, sort_order, set_function, 
        use_function, date_added) '
			. "VALUES ('Paid Order Status', 'MODULE_PAYMENT_ZEN_PAID_STATUS_ID', '2', 
            'Automatically set the status of paid orders to this value.', '6', '0', 
            'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");

		$db->Execute('INSERT INTO ' . TABLE_CONFIGURATION . ' (configuration_title, configuration_key, 
        configuration_value, configuration_description, configuration_group_id, sort_order, set_function, 
        use_function, date_added) '
			. "VALUES ('Refunded Order Status', 'MODULE_PAYMENT_ZEN_REFUNDED_STATUS_ID', '2', 
            'Automatically set the status of fully refunded orders to this value.', '6', '0', 
            'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");

		$db->Execute('INSERT INTO ' . TABLE_CONFIGURATION . ' (configuration_title, configuration_key, 
        configuration_value, configuration_description, configuration_group_id, sort_order, use_function, 
        set_function, date_added) '
			. "VALUES ('Payment Zone', 'MODULE_PAYMENT_ZEN_ZONE', '0', 
            'If a zone is selected, only enable this payment method for that zone.', '6', '2', 
            'zen_get_zone_class_title', 'zen_cfg_pull_down_zone_classes(', now())");

		$db->Execute('INSERT INTO ' . TABLE_CONFIGURATION . ' (configuration_title, configuration_key, 
        configuration_value, configuration_description, configuration_group_id, sort_order, date_added) '
			. "VALUES ('Sort order of display.', 'MODULE_PAYMENT_ZEN_SORT_ORDER', '0', 
            'Sort order of display. Lowest is displayed first.', '6', '2', now())");

		$db->Execute('CREATE TABLE IF NOT EXISTS ' . ZenHelper::ZEN_CONFIRMED_TRANSACTION_TABLE_NAME
			. ' (`id` INT NOT NULL AUTO_INCREMENT, `order_id` INT NOT NULL, `transaction_id` VARCHAR(50) NOT NULL, 
            PRIMARY KEY (`id`))');

		$db->Execute('CREATE TABLE IF NOT EXISTS ' . ZenHelper::ZEN_REFUNDS_HISTORY_TABLE_NAME
			. ' (`id` INT NOT NULL AUTO_INCREMENT, `order_id` INT NOT NULL, `refund_amount` VARCHAR(10) NOT NULL, 
            `refund_date` DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP, PRIMARY KEY (`id`))');
	}

	/**
	 * remove module config from database.
	 */
	public function remove()
	{
		global $db;
		$db->Execute('DELETE FROM ' . TABLE_CONFIGURATION
			. " WHERE configuration_key IN ('" . implode("', '", $this->keys()) . "')");
		$db->Execute('DROP TABLE IF EXISTS ' . ZenHelper::ZEN_CONFIRMED_TRANSACTION_TABLE_NAME);
	}

	/**
	 * return module configuration keys.
	 *
	 * @return array
	 */
	public function keys()
	{
		return [
			'MODULE_PAYMENT_ZEN_STATUS',
			'MODULE_PAYMENT_ZEN_ENVIRONMENT',
			'MODULE_PAYMENT_ZEN_TERMINAL_UUID',
			'MODULE_PAYMENT_ZEN_PAYWALL_SECRET',
			'MODULE_PAYMENT_ZEN_IPN_SECRET',
			'MODULE_PAYMENT_ZEN_TERMINAL_API_KEY',
			'MODULE_PAYMENT_ZEN_UNPAID_STATUS_ID',
			'MODULE_PAYMENT_ZEN_PAID_STATUS_ID',
			'MODULE_PAYMENT_ZEN_REFUNDED_STATUS_ID',
			'MODULE_PAYMENT_ZEN_ZONE',
			'MODULE_PAYMENT_ZEN_SORT_ORDER',
		];
	}

	/**
	 * @param $refunds
	 * @param $refunded_amount
	 *
	 * @return string
	 */
	private function createRefundHistoryTable($refunds, $refunded_amount)
	{
		$refundHistoryTable = '<table style="margin-top: 30px; width: 400px" border="1" cellspacing="0" cellpadding="5">
          <caption><strong>Refund History: </strong></caption>
          <tbody>
            <tr>
                <td align="center"><strong>Refund Date</strong></td>
                <td align="center"><strong>Amount</strong></td>
            </tr>';

		while(!$refunds->EOF) {
			$refundHistoryTable .= '<tr>
                <td align="center"><strong>' . $refunds->fields['refund_date'] . '</strong></td>
                <td align="center"><strong>' . ZenHelper::getValidAmount($refunds->fields['refund_amount'])
				. '</strong></td></tr>';
			$refunds->MoveNext();
		}

		$refundHistoryTable .= '<tr><td align="right"><strong>Total:</strong></td>
                <td align="center"><strong>' . ZenHelper::getValidAmount($refunded_amount) . '</strong></td></tr>
                </tbody></table><br />';

		return $refundHistoryTable;
	}

	/**
	 * @return string
	 */
	private function createRefundForm()
	{

		$refundForm = '<td><table class="noprint">';
		$refundForm .= '<tr style="background-color : #eeeeee; border-style : dotted;">';
		$refundForm .= '<td class="main">' . MODULE_PAYMENT_ZEN_ENTRY_REFUND_TITLE . '<br />';
		$refundForm .= zen_draw_form('zen_refund', FILENAME_ORDERS, zen_get_all_get_params(['action'])
				. 'action=doRefund', 'post', '', true) . zen_hide_session_id();
		$refundForm .= MODULE_PAYMENT_ZEN_ENTRY_REFUND_FULL;
		$refundForm .= '<br /><input type="submit" name="fullrefund" value="'
			. MODULE_PAYMENT_ZEN_ENTRY_REFUND_BUTTON_TEXT_FULL . '" title="'
			. MODULE_PAYMENT_ZEN_ENTRY_REFUND_BUTTON_TEXT_FULL . '" />' . ' '
			. MODULE_PAYMENT_ZEN_TEXT_REFUND_FULL_CONFIRM_CHECK
			. zen_draw_checkbox_field('reffullconfirm', '', false) . '<br /><br />';
		$refundForm .= MODULE_PAYMENT_ZEN_ENTRY_PARTIAL_REFUND_TITLE;
		$refundForm .= MODULE_PAYMENT_ZEN_ENTRY_REFUND_TEXT_FULL . ' <br />'
			. zen_draw_input_field('refund_amount', 'Enter amount', 'length="8"');
		$refundForm .= '<input type="submit" name="partialrefund" value="'
			. MODULE_PAYMENT_ZEN_ENTRY_REFUND_BUTTON_TEXT_PARTIAL
			. '" title="' . MODULE_PAYMENT_ZEN_ENTRY_REFUND_BUTTON_TEXT_PARTIAL . '" /><br />';
		$refundForm .= '</form>';
		$refundForm .= '</td></tr></table></td><br />';

		return $refundForm;
	}
}
