<?php

class ZenHelper
{
    const ZEN_CONFIRMED_TRANSACTION_TABLE_NAME = 'zen_confirmed_transactions';
    const ZEN_REFUNDS_HISTORY_TABLE_NAME       = 'zen_refunds_history';

    /**
     * @param $orderId
     * @return queryFactoryResult
     */
    public static function getOrderById($orderId)
    {
        global $db;

        $sql = 'SELECT * FROM ' . TABLE_ORDERS . ' WHERE orders_id = :id';
        $sql = $db->bindVars($sql, ':id', $orderId, 'integer');

        return $db->Execute($sql);
    }

	/**
	 * Return price in correct format.
	 *
	 * @param $amount
	 * @return float
	 */
	public static function getValidAmount($amount)
	{
		return number_format((float) $amount, 2, '.', '');
	}

    /**
     * @param $orderId
     * @return queryFactoryResult
     */
    public static function getTransactionByOrderId($orderId)
    {
        global $db;

        $sql = 'SELECT * FROM ' . self::ZEN_CONFIRMED_TRANSACTION_TABLE_NAME
            . ' WHERE order_id = :orderId';
        $sql = $db->bindVars($sql, ':orderId', $orderId, 'integer');

        return $db->Execute($sql);
    }

    /**
     * @param $orderId
     * @param $transactionId
     */
    public static function addTransactionConfirmation($orderId, $transactionId)
    {
        global $db;

        $sql = 'INSERT INTO ' . self::ZEN_CONFIRMED_TRANSACTION_TABLE_NAME
            . ' (order_id, transaction_id) VALUES (:orderId, :transactionId)';
        $sql = $db->bindVars($sql, ':orderId', $orderId, 'integer');
        $sql = $db->bindVars($sql, ':transactionId', $transactionId, 'string');
        $db->Execute($sql);
    }

    /**
     * @param $orderId
     * @param $refundAmount
     */
    public static function addRefundConfirmation($orderId, $refundAmount)
    {
        global $db;

        $sql = 'INSERT INTO ' . self::ZEN_REFUNDS_HISTORY_TABLE_NAME
            . ' (order_id, refund_amount) VALUES (:orderId, ":refundAmount")';
        $sql = $db->bindVars($sql, ':orderId', $orderId, 'integer');
        $sql = $db->bindVars($sql, ':refundAmount', self::getValidAmount($refundAmount), 'float');
        $db->Execute($sql);
    }

    /**
     * @param $orderId
     * @param $orderStatus
     */
    public static function updateOrderStatus($orderId, $orderStatus)
    {
        global $db;

        $sql = 'UPDATE ' . TABLE_ORDERS . ' SET orders_status = :status WHERE orders_id = :orderId';
        $sql = $db->bindVars($sql, ':status', $orderStatus, 'integer');
        $sql = $db->bindVars($sql, ':orderId', $orderId, 'integer');
        $db->Execute($sql);
    }

    /**
     * @param $orderId
     * @return queryFactoryResult
     */
    public static function getRefundHistoryForOrder($orderId)
    {
        global $db;

        $sql = 'SELECT * FROM ' . ZenHelper::ZEN_REFUNDS_HISTORY_TABLE_NAME . ' WHERE order_id = :id';
        $sql = $db->bindVars($sql, ':id', $orderId, 'integer');

        return $db->Execute($sql);
    }
}
