<?php

use Zen\Payment\Notification;

require_once 'ZenHelper.php';
require_once 'payment-core/src/Notification.php';
require_once 'payment-core/src/Util.php';
require_once 'payment-core/src/Validate.php';
require_once 'payment-core/vendor/autoload.php';

class ZenIpn
{

	const OS_PENDING = '1';
	const OS_PROCESSING = '2';

	public function notification()
	{

		$notification = new Notification(
			MODULE_PAYMENT_ZEN_IPN_SECRET
		);

		$resultCheckRequestNotification = $notification->checkRequest();

		if(is_int($resultCheckRequestNotification)) {

			echo Notification::formatResponse(Notification::NS_ERROR);
			exit;
		}

		$order = ZenHelper::getOrderById($resultCheckRequestNotification['merchantTransactionId']);
		$currencyValue = (float) $order->fields['currency_value'];
		$orderTotal = (float) $order->fields['order_total'];

		if(!$order) {
			echo Notification::formatResponse(Notification::NS_ERROR);
			exit;
		}

		switch($resultCheckRequestNotification['status']) {
			case Notification::STATUS_ACCEPTED:
				$notificationStatus = Notification::STATUS_ACCEPTED;
				$notificationTransactionStatus = MODULE_PAYMENT_ZEN_PAID_STATUS_ID;
				break;
			case Notification::STATUS_REJECTED:
				$notificationStatus = Notification::STATUS_REJECTED;
				$notificationTransactionStatus = MODULE_PAYMENT_ZEN_UNPAID_STATUS_ID;
				break;
			default:
				echo Notification::formatResponse(Notification::NS_OK);
				exit;
		}

		if($resultCheckRequestNotification['type'] === Notification::TRT_REFUND) {

			if($notificationStatus !== Notification::STATUS_ACCEPTED) {
				echo Notification::formatResponse(Notification::NS_ERROR);
				exit;
			}

			$refunded = $this->getRefundedAmountForOrder($resultCheckRequestNotification['merchantTransactionId']);

			if(((float) $refunded + (float) $resultCheckRequestNotification['amount']) > ($orderTotal * (float) $order->fields['currency_value'])) {
				echo Notification::formatResponse(Notification::NS_ERROR);
				exit;
			}

			ZenHelper::updateOrderStatus($resultCheckRequestNotification['merchantTransactionId'], MODULE_PAYMENT_ZEN_REFUNDED_STATUS_ID);
			ZenHelper::addRefundConfirmation($resultCheckRequestNotification['merchantTransactionId'], $resultCheckRequestNotification['amount']);

			echo Notification::formatResponse(Notification::NS_OK);
			exit;
		}

		if($order->fields['orders_status'] !== self::OS_PENDING) {
			echo Notification::formatResponse(Notification::NS_ERROR);
			exit;
		}

		if(!$notification::checkRequestAmount($resultCheckRequestNotification, round($orderTotal * $currencyValue, 2), $order->fields['currency'])) {
			echo Notification::formatResponse(Notification::NS_ERROR);
			exit;
		}

		ZenHelper::updateOrderStatus($resultCheckRequestNotification['merchantTransactionId'], $notificationTransactionStatus);
		ZenHelper::addTransactionConfirmation($resultCheckRequestNotification['merchantTransactionId'], $resultCheckRequestNotification['transactionId']);

		echo Notification::formatResponse(Notification::NS_OK);
		exit;
	}

	/**
	 * @param $orderId
	 *
	 * @return string
	 */
	public function getRefundedAmountForOrder($orderId)
	{
		global $db;

		$sql = 'SELECT SUM(`refund_amount`) AS `refunded` FROM ' . ZenHelper::ZEN_REFUNDS_HISTORY_TABLE_NAME
			. ' WHERE `order_id` = :id';
		$sql = $db->bindVars($sql, ':id', $orderId, 'integer');
		$refundsAmountSum = $db->Execute($sql);

		return $refundsAmountSum->fields['refunded'];
	}

}
